/*
 * Copyright (c) 2019 Hanspeter Portner (dev@open-music-kontrollers.ch)
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the Artistic License 2.0 as published by
 * The Perl Foundation.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License 2.0 for more details.
 *
 * You should have received a copy of the Artistic License 2.0
 * along the source as a COPYING file. If not, obtain it from
 * http://www.perlfoundation.org/artistic_license_2_0.
 */

#ifndef _JIT_API_H
#define _JIT_API_H

#include <stdint.h>
#include <stdbool.h>

#define JIT_UNUSED   __attribute__((unused))
#define JIT_EXPORT   __attribute__((visibility("default")))

#define JIT_VERSION_MAJOR 0
#define JIT_VERSION_MINOR 1
#define JIT_VERSION_MICRO 301

#define JIT_MAX_VOICES 64

#if 0
#if ( \
		(0 > JIT_VERSION_MAJOR) \
	) || ( \
		(0 == JIT_VERSION_MAJOR) \
		&& (1 > JIT_VERSION_MINOR) \
	) || ( \
		(0 == JIT_VERSION_MAJOR) \
		&& (1 == JIT_VERSION_MINOR) \
		&& (301 > JIT_VERSION_MICRO) \
	)
#	error "Your installed version of jit.lv2 is too old for this script"
#endif
#endif

typedef enum _jit_ev_type_t {
	JIT_EVENT_ON,
	JIT_EVENT_OFF,
	JIT_EVENT_SET
} jit_ev_type_t ;

typedef enum _jit_mask_t {
	JIT_MASK_NONE           = 0,

	JIT_MASK_SRATE          = (1 << 0),
	JIT_MASK_NCHANNELS      = (1 << 1),
	JIT_MASK_NCONTROLS      = (1 << 2),

	JIT_MASK_NSAMPLES       = (1 << 3),
	JIT_MASK_AUDIO_IN       = (1 << 4),
	JIT_MASK_AUDIO_OUT      = (1 << 5),
	JIT_MASK_CONTROL        = (1 << 6),

	JIT_MASK_ASSERT         = (1 << 7),
	JIT_MASK_VPRINTF        = (1 << 8),
	JIT_MASK_WORK_WRITE_REQ = (1 << 9),
	JIT_MASK_WORK_WRITE_ADV = (1 << 10),
	JIT_MASK_WORK_READ_REQ  = (1 << 11),
	JIT_MASK_WORK_READ_ADV  = (1 << 12),

	JIT_MASK_MAX            = (1 << 13)
} jit_mask_t ;

#define JIT_MASK_ALL (JIT_MASK_MAX - 1)

typedef struct _jit_assert_t jit_assert_t;
typedef struct _jit_print_t jit_print_t;
typedef struct _jit_work_write_request_t jit_work_write_request_t;
typedef struct _jit_work_write_advance_t jit_work_write_advance_t;
typedef struct _jit_work_read_request_t jit_work_read_request_t;
typedef struct _jit_work_read_advance_t jit_work_read_advance_t;

typedef struct _jit_t jit_t;
typedef struct _jit_ev_t jit_ev_t;
typedef struct _jit_desc_t jit_desc_t;

typedef void *(*jit_init_t)(jit_t *const jit);
typedef void  (*jit_deinit_t)(jit_t *const, void *data);
typedef void  (*jit_run_t)(jit_t *const jit, void *const data);
typedef void  (*jit_event_t)(jit_t *const jit, void *data,
	jit_ev_t const *const ev);
typedef void  (*jit_work_t)(jit_t *const jit, void *data);

typedef int   (*jit_print_func_t)(void *const data, const char *fmt);
typedef void  (*jit_assert_func_t)(void *const data);
typedef void *(*jit_work_write_request_func_t)(void *const data, size_t len);
typedef void  (*jit_work_write_advance_func_t)(void *const data, size_t len);
typedef void const*(*jit_work_read_request_func_t)(void *const data, size_t *len);
typedef void  (*jit_work_read_advance_func_t)(void *const data);

struct _jit_assert_t {
	void *const data;
	jit_assert_func_t func;
};

struct _jit_print_t {
	void *const data;
	jit_print_func_t func;
};

struct _jit_work_write_request_t {
	void *const data;
	jit_work_write_request_func_t func;
};

struct _jit_work_write_advance_t {
	void *const data;
	jit_work_write_advance_func_t func;
};

struct _jit_work_read_request_t {
	void *const data;
	jit_work_read_request_func_t func;
};

struct _jit_work_read_advance_t {
	void *const data;
	jit_work_read_advance_func_t func;
};

struct _jit_t {
	// public data
	uint32_t mask;

	uint32_t srate;
	uint32_t nchannels;
	uint32_t ncontrols;

	uint32_t nsamples;
	float const *const *const audio_in;
	float *const *const audio_out;
	float const *const control;

	// private data
	jit_assert_t _assert;
	jit_print_t _print;
	jit_work_write_request_t _work_write_request;
	jit_work_write_advance_t _work_write_advance;
	jit_work_read_request_t _work_read_request;
	jit_work_read_advance_t _work_read_advance;
};

struct _jit_ev_t {
	uint32_t etype;
	uint32_t index;
	float gain;
	float freq;
	float pressure;
	float timbre;
};

struct _jit_desc_t {
	jit_init_t init;
	jit_deinit_t deinit;
	jit_run_t run;
	jit_event_t event;
	jit_work_t work;
};

#endif // _JIT_API_H
