/*
 * Copyright (c) 2019 Hanspeter Portner (dev@open-music-kontrollers.ch)
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the Artistic License 2.0 as published by
 * The Perl Foundation.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License 2.0 for more details.
 *
 * You should have received a copy of the Artistic License 2.0
 * along the source as a COPYING file. If not, obtain it from
 * http://www.perlfoundation.org/artistic_license_2_0.
 */

#ifndef _JIT_UTIL_H
#define _JIT_UTIL_H

#include <stdio.h>
#include <stdarg.h>

#include <jit_api.h>

/****f* jit.lv2/jit_assert
 * SYNOPSIS
 */
static inline void
jit_assert(jit_t *const jit, bool condition)
/*
 * FUNCTION
 *  If condition is false, makes a longjump back to wrapping function start.
 * INPUTS
 *  - jit       Pointer to opaque jit structure
 *  - condition	Condition, whether to longjump or not
 ****
 */
{
	if(!condition && (jit->mask & JIT_MASK_ASSERT) && jit->_assert.func)
	{
		jit->_assert.func(jit->_assert.data);
	}
}

/****f* jit.lv2/jit_vprintf
 * SYNOPSIS
 */
static inline int
jit_vprintf(jit_t *const jit, const char *fmt, va_list ap)
/*
 * FUNCTION
 *  Formatted print to LV2 host log infrastructure.
 * INPUTS
 *  - jit	Pointer to opaque jit structure
 *  - fmt	Format string
 *  - ap	List of argument
 * OUTPUT
 *  Upon successful return, this function returns the number of characters
 *  printed. If an output error is encountered, a negative value is returned.
 ****
 */
{
	char buf [512];

	vsnprintf(buf, sizeof(buf), fmt, ap);

	if( (jit->mask & JIT_MASK_VPRINTF) && jit->_print.func)
	{
		return jit->_print.func(jit->_print.data, buf);
	}

	return -1;
}

static inline int __attribute__((format(printf, 2, 3)))
jit_printf(jit_t *const jit, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	const int ret = jit_vprintf(jit, fmt, ap);
	va_end(ap);

	return ret;
}

static inline void *
jit_work_write_request(jit_t *const jit, size_t len)
{
	if( (jit->mask & JIT_MASK_WORK_WRITE_REQ) && jit->_work_write_request.func)
	{
		return jit->_work_write_request.func(jit->_work_write_request.data, len);
	}

	return NULL;
}

static inline void
jit_work_write_advance(jit_t *const jit, size_t len)
{
	if( (jit->mask & JIT_MASK_WORK_WRITE_ADV) && jit->_work_write_advance.func)
	{
		jit->_work_write_advance.func(jit->_work_write_advance.data, len);
	}
}

static inline void const*
jit_work_read_request(jit_t *const jit, size_t *len)
{
	if( (jit->mask & JIT_MASK_WORK_READ_REQ) && jit->_work_read_request.func)
	{
		return jit->_work_read_request.func(jit->_work_read_request.data, len);
	}

	return NULL;
}

static inline void
jit_work_read_advance(jit_t *const jit)
{
	if( (jit->mask & JIT_MASK_WORK_READ_ADV) && jit->_work_read_advance.func)
	{
		jit->_work_read_advance.func(jit->_work_read_advance.data);
	}
}

#endif // _JIT_UTIL_H
